/**
 * An element which displays the tags panel for a repository view.
 */
angular.module('quay').directive('repoPanelTags', function () {
  var directiveDefinitionObject = {
    priority: 0,
    templateUrl: '/static/directives/repo-view/repo-panel-tags.html',
    replace: false,
    transclude: false,
    restrict: 'C',
    scope: {
      'repository': '=repository',
      'selectedTags': '=selectedTags',
      'imagesResource': '=imagesResource',
      'images': '=images',
    },
    controller: function($scope, $element, $filter, $location, ApiService, UIService) {
      var orderBy = $filter('orderBy');

      $scope.checkedTags = UIService.createCheckStateController([]);
      $scope.options = {
        'predicate': 'last_modified_datetime',
        'reverse': false
      };

      $scope.iterationState = {};
      $scope.tagActionHandler = null;
      $scope.showingHistory = false;

      var setTagState = function() {
        if (!$scope.repository || !$scope.selectedTags) { return; }

        var tags = [];
        var allTags = [];
        var imageMap = {};
        var imageTracks = [];

        // Build a list of tags and filtered tags.
        for (var tag in $scope.repository.tags) {
          if (!$scope.repository.tags.hasOwnProperty(tag)) { continue; }

          var tagData = $scope.repository.tags[tag];
          var tagInfo = $.extend(tagData, {
            'name': tag,
            'last_modified_datetime': (new Date(tagData.last_modified || 0)).valueOf() * (-1)
          });

          allTags.push(tagInfo);

          if (!$scope.options.tagFilter || tag.indexOf($scope.options.tagFilter) >= 0 ||
              tagInfo.image_id.indexOf($scope.options.tagFilter) >= 0) {
            tags.push(tagInfo);
          }
        }

        // Sort the tags by the predicate and the reverse, and map the information.
        var imageIDs = [];
        var ordered = orderBy(tags, $scope.options.predicate, $scope.options.reverse);
        var checked = [];

        for (var i = 0; i < ordered.length; ++i) {
          var tagInfo = ordered[i];
          if (!imageMap[tagInfo.image_id]) {
            imageMap[tagInfo.image_id] = [];
            imageIDs.push(tagInfo.image_id)
          }

          imageMap[tagInfo.image_id].push(tagInfo);

          if ($.inArray(tagInfo.name, $scope.selectedTags) >= 0) {
            checked.push(tagInfo);
          }
        };

        // Calculate the image tracks.
        var colors = d3.scale.category10();
        var index = 0;

        imageIDs.sort().map(function(image_id) {
          if (imageMap[image_id].length >= 2){
            imageTracks.push({
              'image_id': image_id,
              'color': colors(index),
              'count': imageMap[image_id].length,
              'tags': imageMap[image_id]
            });
            ++index;
          }
        });

        $scope.imageMap = imageMap;
        $scope.imageTracks = imageTracks;

        $scope.tags = ordered;
        $scope.allTags = allTags;

        $scope.checkedTags = UIService.createCheckStateController(ordered, checked);
        $scope.checkedTags.listen(function(checked) {
          $scope.selectedTags = checked.map(function(tag_info) {
            return tag_info.name;
          });
        });
      }

      $scope.$watch('options.predicate', setTagState);
      $scope.$watch('options.reverse', setTagState);
      $scope.$watch('options.tagFilter', setTagState);

      $scope.$watch('selectedTags', function(selectedTags) {
        if (!selectedTags || !$scope.repository || !$scope.imageMap) { return; }

        $scope.checkedTags.checked = selectedTags.map(function(tag) {
          return $scope.repository.tags[tag];
        });
      }, true);

      $scope.$watch('repository', function(repository) {
        if (!repository) { return; }

        // Process each of the tags.
        setTagState();

        if ($scope.showingHistory) {
          loadTimeline();
        }
      });

      var loadTimeline = function() {
        var params = {
          'repository': $scope.repository.namespace + '/' + $scope.repository.name
        };

        ApiService.listRepoTags(null, params).then(function(resp) {
          var tagData = [];
          var currentTags = {};

          resp.tags.forEach(function(tag) {
            var tagName = tag.name;
            var imageId = tag.docker_image_id;
            var oldImageId = null;

            if (tag.end_ts) {
              var action = 'delete';

              // If the end time matches the existing start time for this tag, then this is a move
              // instead of a delete.
              var currentTime = tag.end_ts * 1000;
              if (currentTags[tagName] && currentTags[tagName].start_ts == tag.end_ts) {
                action = 'move';

                // Remove the create.
                var index = tagData.indexOf(currentTags[tagName]);
                var createEntry = tagData.splice(index, 1)[0];

                imageId = createEntry.docker_image_id;
                oldImageId = tag.docker_image_id;
              }

              // Add the delete/move.
              tagData.push({
                'tag_name': tagName,
                'action': action,
                'start_ts': tag.start_ts,
                'end_ts': tag.end_ts,
                'time': currentTime,
                'docker_image_id': imageId,
                'old_docker_image_id': oldImageId || ''
              })
            }

            if (tag.start_ts) {
              var currentTime = tag.start_ts * 1000;
              var create = {
                'tag_name': tagName,
                'action': 'create',
                'start_ts': tag.start_ts,
                'end_ts': tag.end_ts,
                'time': currentTime,
                'docker_image_id': tag.docker_image_id,
                'old_docker_image_id': ''
              };

              tagData.push(create);
              currentTags[tagName] = create;
            }
          });

          tagData.sort(function(a, b) {
            return b.time - a.time;
          });

          for (var i = tagData.length - 1; i >= 1; --i) {
            var current = tagData[i];
            var next = tagData[i - 1];

            if (new Date(current.time).getDate() != new Date(next.time).getDate()) {
              tagData.splice(i - 1, 0, {
                'date_break': true,
                'date': new Date(current.time)
              });
              i--;
            }
          }

          if (tagData.length > 0) {
            tagData.splice(0, 0, {
              'date_break': true,
              'date': new Date(tagData[0].time)
            });
          }

          $scope.tagHistoryData = tagData;
        });
      };

      $scope.getEntryClasses = function(entry, historyFilter) {
        var classes = entry.action + ' ';
        if (!historyFilter || !entry.action) {
          return classes;
        }

        var parts = (historyFilter || '').split(',');
        var isMatch = parts.some(function(part) {
          if (part && entry.tag_name) {
            isMatch = entry.tag_name.indexOf(part) >= 0;
            isMatch = isMatch || entry.docker_image_id.indexOf(part) >= 0;
            isMatch = isMatch || entry.old_docker_image_id.indexOf(part) >= 0;
            return isMatch;
          }
        });

        classes += isMatch ? 'filtered-match' : 'filtered-mismatch';
        return classes;
      };

      $scope.showHistory = function(value, opt_tagname) {
        if (opt_tagname) {
          $scope.options.historyFilter = opt_tagname;
        } else {
          $scope.options.historyFilter = '';
        }

        if ($scope.showingHistory == value) {
          return;
        }

        $scope.showingHistory = value;

        if ($scope.showingHistory) {
          loadTimeline();
        }
      };

      $scope.toggleHistory = function() {
        $scope.showHistory(!$scope.showingHistory);
      };

      $scope.trackLineClass = function(index, track_info) {
        var startIndex = $.inArray(track_info.tags[0], $scope.tags);
        var endIndex = $.inArray(track_info.tags[track_info.tags.length - 1], $scope.tags);

        if (index == startIndex) {
          return 'start';
        }

        if (index == endIndex) {
          return 'end';
        }

        if (index > startIndex && index < endIndex) {
          return 'middle';
        }

        if (index < startIndex) {
          return 'before';
        }

        if (index > endIndex) {
          return 'after';
        }
      };

      $scope.tablePredicateClass = function(name, predicate, reverse) {
        if (name != predicate) {
          return '';
        }

        return 'current ' + (reverse ? 'reversed' : '');
      };

      $scope.askDeleteTag = function(tag) {
        $scope.tagActionHandler.askDeleteTag(tag);
      };

      $scope.askDeleteMultipleTags = function(tags) {
        if (tags.length == 1) {
          $scope.askDeleteTag(tags[0].name);
          return;
        }

        $scope.tagActionHandler.askDeleteMultipleTags(tags);
      };

      $scope.askAddTag = function(tag) {
        $scope.tagActionHandler.askAddTag(tag.image_id);
      };

      $scope.orderBy = function(predicate) {
        if (predicate == $scope.options.predicate) {
          $scope.options.reverse = !$scope.options.reverse;
          return;
        }

        $scope.options.reverse = false;
        $scope.options.predicate = predicate;
      };

      $scope.commitTagFilter = function(tag) {
        var r = new RegExp('^[0-9a-f]{7}$');
        return tag.name.match(r);
      };

      $scope.allTagFilter = function(tag) {
        return true;
      };

      $scope.noTagFilter = function(tag) {
        return false;
      };

      $scope.imageIDFilter = function(image_id, tag) {
        return tag.image_id == image_id;
      };

      $scope.setTab = function(tab) {
        $location.search('tab', tab);
      };

      $scope.selectTrack = function(it) {
        $scope.checkedTags.checkByFilter(function(tag) {
          return $scope.imageIDFilter(it.image_id, tag);
        });
      };

      $scope.getTagNames = function(checked) {
        var names = checked.map(function(tag) {
          return tag.name;
        });

        return names.join(',');
      };

      $scope.isChecked = function(tagName, checked) {
        return checked.some(function(tag) {
          if (tag.name == tagName) {
            return true;
          }
        });
      };
    }
  };
  return directiveDefinitionObject;
});

